import {
  Text, Progress, ApexChart,
} from '@zeedhi/common';
import {
  IDictionary, Http, Metadata,
  Loader,
} from '@zeedhi/core';
import {
  IterableComponentRender,
} from '@zeedhi/teknisa-components-common';

export class DashboardController {
  public hierarquiasController = Loader.getInstance('HierarquiasController');

  public slotValue: number | string = '60%';

  // Define as porcentagens das "Tarefas Conluídas"
  public async percent() {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray(); // Busca os usuários subordinados do usuário logado

    Http.post('/percentageTasks', { usersIds: hierarchyUsersList }).then((response: IDictionary) => {
      const data = response.data.dataset.percents.flat(3);
      Metadata.getInstance<Text>('textTaskCompleted').text = `Tarefas Concluídas: ${data[1].QTDTAREFASCONCLUIDAS}/${data[0].QTDTAREFAS}`;
      const percent = (data[0].QTDTAREFAS === 0) ? (0) : (Math.floor((data[1].QTDTAREFASCONCLUIDAS * 100) / data[0].QTDTAREFAS));
      Metadata.getInstance<Progress>('slotPercentage').value = `${percent}%`;
      Metadata.getInstance<Text>('percentageText').text = `${percent}%`;
    });
  }

  // Criar os cards "Tarefas Concluídas"
  public async createCardTaskCompleted() {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray(); // Busca os usuários subordinados do usuário logado

    Http.post('/tasksCompleted', { usersIds: hierarchyUsersList }).then((response: IDictionary) => {
      response.data.dataset.tasks.forEach((dados: IDictionary) => {
        const DateTime = dados.DTULTATU.split(' ');
        Metadata.getInstance<IterableComponentRender>('iCardTasksCompleted').datasource.data.push({
          NR: dados.NRCHECKLISTAGEND,
          NMTAREFA: dados.NMCHECKLISTAGEND,
          NAMELOCAL: dados.NMLOCALCLIENTE,
          NAMETYPELOCAL: dados.NMLOCALPAI,
          TIMEFINISHEDTASK: DateTime[1],
          DATEFINISHEDTASK: DateTime[0],
          EMPLOYEENAME: dados.NMFUNCIONARIO,
        });
      });
    });
  }

  // Criar os cards "Tarefas Atrasadas"
  public async createCardTaskLate() {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray(); // Busca os usuários subordinados do usuário logado

    Http.post('/tasksLate', { usersIds: hierarchyUsersList }).then((response: IDictionary) => {
      response.data.dataset.tasksLate.forEach((dados: IDictionary) => {
        const DateTime = dados.DTHORAAGEND.split(' ');
        Metadata.getInstance<IterableComponentRender>('iCardTasksLate').datasource.data.push({
          NR: dados.NRCHECKLISTAGEND,
          NMTAREFA: dados.NMCHECKLISTAGEND,
          NAMELOCAL: dados.NMLOCALCLIENTE,
          NAMETYPELOCAL: dados.NMLOCALPAI,
          DTHORAATRASO: dados.DTHORAATRASO,
          DATETASK: DateTime[0],
          EMPLOYEENAME: dados.NMFUNCIONARIO,
        });
      });
    });
  }

  // Criar os cards "Limpeza Finalizada"
  public async createCardFinishedTasks() {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray(); // Busca os usuários subordinados do usuário logado

    Http.post('/finishedTasks', { usersIds: hierarchyUsersList }).then((response: IDictionary) => {
      response.data.dataset.finishedTasks.forEach((dados: IDictionary) => {
        const DateTime = dados.DTULTATU.split(' ');
        Metadata.getInstance<IterableComponentRender>('iCardFinishedTasks').datasource.data.push({
          NR: dados.NRCHECKLISTAGEND,
          NMTAREFA: dados.NMCHECKLISTAGEND,
          NAMELOCAL: dados.NMLOCALCLIENTE,
          NAMETYPELOCAL: dados.NMLOCALPAI,
          TIMEFINISHEDTASK: DateTime[1],
          DATEFINISHEDTASK: DateTime[0],
          EMPLOYEENAME: dados.NMFUNCIONARIO,
        });
      });
    });
  }

  // -------------------> FUNÇÕES DA TELA DE DASHBOARD DO SUPERVISOR <-------------------
  //
  public async createCardLastTasksPerformade() { // Criar os cards "Ultimas tarefas realizadas"
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray(); // Busca os usuários subordinados do usuário logado

    Http.post('/lastTasksPerformed', { usersIds: hierarchyUsersList }).then((response: IDictionary) => {
      response.data.dataset.lastTask.forEach((dados: IDictionary) => {
        const DateTime = dados.DTULTATU.split(' ');
        Metadata.getInstance<IterableComponentRender>('iCardLastTasks').datasource.data.push({
          NR: dados.CODIGO,
          NMTAREFA: dados.NOME,
          LOCALNAME: dados.NMLOCALCLIENTE,
          NAMETYPELOCAL: dados.NMLOCALPAI,
          TIMEFINISHEDTASK: DateTime[1],
          DATEFINISHEDTASK: DateTime[0],
          EMPLOYEENAME: dados.NMFUNCIONARIO,
        });
      });
    });
  }

  public async dataBaseChartPie({ component }:{ component: ApexChart }) {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray(); // Busca os usuários subordinados do usuário logado

    await Http.post('/dataBaseChartPie', { usersIds: hierarchyUsersList }).then((response: IDictionary) => {
      if (response.data.dataset.dbChartp.length) {
        response.data.dataset.dbChartp.forEach((dados: IDictionary) => {
          component.options.labels.push(dados.NMATIVIDADE);
          (component.series as any[]).push(dados.QUANTIDADE);
          component.options.colors.push(dados.DSCORIDATIVIDADE);
        });
      } else {
        component.options.labels.push('Nenhum dado disponível');
        (component.series as any[]).push(100);
        component.options.colors.push('#57737a');
      }
    });
  }

  public async dataBaseChartBar({ component }:{ component: ApexChart }) {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray(); // Busca os usuários subordinados do usuário logado

    Http.post('/dataBaseChartBar', { usersIds: hierarchyUsersList }).then((response: IDictionary) => {
      if (response.data.dataset.dbChartb.length) {
        response.data.dataset.dbChartb.forEach((dados: IDictionary) => {
          (component.series as any[]).push({
            name: dados.NMATIVIDADE,
            data: [dados.QUANTIDADE],
          });
          component.options.colors.push(dados.DSCORIDATIVIDADE);
        });
      } else {
        (component.series as any[]).push({
          name: 'Nenhum dado disponível',
          data: [100],
        });
        component.options.colors.push('#57737a');
      }
    });
  }
}
